#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

from fenrirscreenreader.core.i18n import _
from fenrirscreenreader.core.tabCompletionManager import TabCompletionManager


class command:
    def __init__(self):
        self.manager = TabCompletionManager()

    def initialize(self, environment):
        self.env = environment
        self.manager.initialize(environment)

    def shutdown(self):
        pass

    def get_description(self):
        return _("Tracks tab keypresses for completion announcements")

    def run(self):
        self.manager.capture_if_tab()

    def set_callback(self, callback):
        pass
