#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

from fenrirscreenreader.core import debug

settings_data = {
    "sound": {
        "enabled": True,
        "driver": "genericDriver",
        "theme": "default",
        "volume": 1.0,
        "generic_play_file_command": "play -q -v fenrirVolume fenrirSoundFile",
        "generic_frequency_command": "play -q -v fenrirVolume -n -c1 synth fenrirDuration sine fenrirFrequence",
        "progress_monitoring": True,
    },
    "speech": {
        "enabled": True,
        "driver": "genericDriver",
        "server_path": "",
        "rate": 0.75,
        "pitch": 0.5,
        "capital_pitch": 0.8,
        "capital_indicator": "pitch",
        "volume": 1.0,
        "module": "",
        "voice": "en-us",
        "language": "",
        "auto_read_incoming": True,
        "read_numbers_as_digits": False,
        "rapid_update_threshold": 5,
        "rapid_update_window": 0.3,
        "batch_flush_interval": 0.5,
        "max_batch_lines": 100,
        "flood_line_threshold": 500,
        "generic_speech_command": 'espeak -a fenrirVolume -s fenrirRate -p fenrirPitch -v fenrirVoice "fenrirText"',
        "fenrir_min_volume": 0,
        "fenrir_max_volume": 200,
        "fenrir_min_pitch": 0,
        "fenrir_max_pitch": 99,
        "fenrir_min_rate": 80,
        "fenrir_max_rate": 450,
    },
    "screen": {
        "driver": "vcsaDriver",
        "encoding": "auto",
        "screen_update_delay": 0.1,
        "ignore_screen": "",
        "autodetect_ignore_screen": False,
    },
    "general": {
        "debug_level": debug.DebugLevel.DEACTIVE,
        "debug_mode": "FILE",
        "debug_file": "",
        "punctuation_profile": "default",
        "punctuation_level": "some",
        "respect_punctuation_pause": True,
        "replace_undefined_punctuation_with_space": True,
        "new_line_pause": True,
        "number_of_clipboards": 10,
        "clipboard_export_path": "/tmp/fenrirClipboard",
        "emoticons": True,
        "fenrir_keys": "KEY_KP0,KEY_META",
        "script_keys": "KEY_COMPOSE",
        "time_format": "%I:%M%P",
        "date_format": "%A, %B %d, %Y",
        "auto_spell_check": False,
        "spell_check_language": "en_US",
        "script_path": "/usr/share/fenrirscreenreader/scripts",
        "command_path": "/usr/share/fenrirscreenreader/commands",
        "attribute_format_string": "Background fenrirBGColor,Foreground fenrirFGColor,fenrirUnderline,fenrirBold,fenrirBlink, Font fenrirFont,Fontsize fenrirFontSize",
        "auto_present_indent": False,
        "auto_present_indent_mode": 1,
        "has_attributes": True,
        "shell": "",
        "diff_presentation": "both",
        "diff_verbosity": "compact",
    },
    "focus": {
        "cursor": True,
        "highlight": False,
        "tui": False,
    },
    "remote": {
        "enabled": True,
        "driver": "unixDriver",
        "port": 22447,
        "socket_file": "",
        "enable_settings_remote": True,
        "enable_command_remote": True,
    },
    "barrier": {
        "enabled": True,
        "left_barriers": "│└┌─",
        "right_barriers": "│┘┐─",
    },
    "review": {
        "line_break": True,
        "end_of_screen": True,
        "leave_review_on_cursor_change": True,
        "leave_review_on_screen_change": True,
    },
    "menu": {
        "vmenu_path": "",
        "quick_menu": "speech#rate;speech#pitch;speech#volume",
    },
    "time": {
        "enabled": False,
        "present_time": True,
        "present_date": True,
        "delay_sec": 0,
        "on_minutes": "00,30",
        "announce": True,
        "interrupt": False,
    },
    "keyboard": {
        "driver": "evdev",
        "device": "all",
        "grab_devices": True,
        "x11_window_id": "",
        "ignore_shortcuts": False,
        "keyboard_layout": "desktop",
        "char_echo_mode": 2,  # while capslock
        "char_delete_echo": True,
        "word_echo": True,
        "interrupt_on_key_press": True,
        "interrupt_on_key_press_filter": "",
        "double_tap_timeout": 0.2,
    },
}
