#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

import string
from collections import OrderedDict

from fenrirscreenreader.core import debug

punctuation_data = {
    "LEVELDICT": {
        "none": "",
        "some": "#-$~+*-/\\@",
        "most": ".,:-$~ +*-/\\@!#%^&*()[]}{<>;",
        "all": string.punctuation + " §",
    },
    "PUNCTDICT": {},
    "CUSTOMDICT": OrderedDict(),
    "EMOTICONDICT": OrderedDict(),
}
