class command():
    def initialize(self, environment):
        self.env = environment
        self.emoji = "🧛"
    
    def shutdown(self):
        pass
    
    def setCallback(self, callback):
        pass
    
    def getDescription(self):
        return "Vampire emoji"
    
    def run(self):
        self.env["runtime"]["MemoryManager"].add_value_to_first_index(
            "clipboardHistory", self.emoji
        )
        self.env["runtime"]["OutputManager"].present_text(
            "Added vampire to clipboard", 
            interrupt=False, flush=False
        )
