#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

from fenrirscreenreader.core import debug

general_data = {
    "args": None,
    "tutorialMode": False,
    "curr_user": "",
    "prev_user": "",
    "managerList": [
        "AttributeManager",
        "PunctuationManager",
        "CursorManager",
        "ApplicationManager",
        "CommandManager",
        "ScreenManager",
        "InputManager",
        "OutputManager",
        "HelpManager",
        "MemoryManager",
        "EventManager",
        "ProcessManager",
        "VmenuManager",
        "QuickMenuManager",
        "ReadAllManager",
        "DiffReviewManager",
        "RemoteManager",
        "SettingsManager",
        "SayAllManager",
        "TableManager",
        "TextManager",
        "BarrierManager",
        "DebugManager",
    ],
    "commandFolderList": [
        "commands",
        "onKeyInput",
        "onCursorChange",
        "onScreenUpdate",
        "onScreenChanged",
        "onHeartBeat",
        "onPlugInputDevice",
        "onApplicationChange",
        "onSwitchApplicationProfile",
        "help",
        "vmenu-navigation",
    ],
}
