#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

import time

from fenrirscreenreader.core import debug

screen_data = {
    "columns": 0,
    "lines": 0,
    "oldDelta": "",
    "oldAttribDelta": "",
    "oldNegativeDelta": "",
    "oldCursorReview": None,
    "oldCursorAttrib": None,
    "old_cursor": {"x": 0, "y": 0},
    "oldContentBytes": b"",
    "old_content_text": "",
    "oldContentAttrib": None,
    "oldApplication": "",
    "oldTTY": None,
    "new_delta": "",
    "new_delta_is_typing": False,
    "newNegativeDelta": "",
    "newAttribDelta": "",
    "newCursorReview": None,
    "newCursorAttrib": None,
    "new_cursor": {"x": 0, "y": 0},
    "newContentBytes": b"",
    "new_content_text": "",
    "newContentAttrib": None,
    "newTTY": "0",
    "new_application": "",
    "lastScreenUpdate": time.time(),
    "autoIgnoreScreens": [],
}
