#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

from fenrirscreenreader.core.i18n import _


class command:
    def __init__(self):
        pass

    def initialize(self, environment):
        self.env = environment

    def shutdown(self):
        pass

    def get_description(self):
        return _(
            "enables or disables automatic reading of new text as it appears"
        )

    def run(self):
        self.env["runtime"]["SettingsManager"].set_setting(
            "speech",
            "auto_read_incoming",
            str(
                not self.env["runtime"]["SettingsManager"].get_setting_as_bool(
                    "speech", "auto_read_incoming"
                )
            ),
        )
        if self.env["runtime"]["SettingsManager"].get_setting_as_bool(
            "speech", "auto_read_incoming"
        ):
            self.env["runtime"]["OutputManager"].present_text(
                _("autoread enabled"), sound_icon="", interrupt=True
            )
        else:
            self.env["runtime"]["OutputManager"].present_text(
                _("autoread disabled"), sound_icon="", interrupt=True
            )

    def set_callback(self, callback):
        pass
