#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.


# this command is just to initialize stuff.
# like init index lists in MemoryManager
# it is not useful to execute it

from fenrirscreenreader.core.i18n import _


class command:
    def __init__(self):
        pass

    def initialize(self, environment):
        self.env = environment
        # clipboard
        self.env["runtime"]["MemoryManager"].add_index_list(
            "clipboardHistory",
            self.env["runtime"]["SettingsManager"].get_setting_as_int(
                "general", "number_of_clipboards"
            ),
        )

    def shutdown(self):
        pass

    def get_description(self):
        return "No description found"

    def run(self):
        pass

    def set_callback(self, callback):
        pass
