#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

from fenrirscreenreader.core import debug

runtime_data = {
    "SpeechDriver": None,
    "ScreenDriver": None,
    "SoundDriver": None,
    "InputDriver": None,
    "RemoteDriver": None,
    "InputManager": None,
    "CommandManager": None,
    "ScreenManager": None,
    "OutputManager": None,
    "DebugManager": None,
    "SettingsManager": None,
    "FenrirManager": None,
    "EventManager": None,
    "ProcessManager": None,
    "DiffReviewManager": None,
}
