#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

from fenrirscreenreader.core import debug


class SayAllManager:
    def __init__(self):
        self.isActive = False
        self.isActiveLock = None

    def initialize(self, environment):
        self.env = environment

    def shutdown(self):
        pass

    def set_is_active(self, isActive):
        pass

    def start(self):
        pass

    def is_say_all_active(self):
        pass

    def say_all_worker(self):
        pass

    def stop(self):
        pass

    def finish(self):
        pass

    def goto_next_page(self):
        pass
